﻿
using HaloMap;
using System.IO;
using LazyIO;
namespace HaloMap
{
    public class MapHeader : LazyIO.LazyIO
    {
        #region layout

        [Size(4)]
        String head;
        Int version;
        Int filesize;
        Int zero;
        Int indexHeaderOffset;
        Int indexHeaderSize;
        Int metaSize;
        Int nonRawSize;
        [Size(32)]
        String mapOrigin;
        [Size(224)]
        Unused unknown1;
        [Size(32)]
        String buildDate;
        MapType mapType;
        public class MapType : Enum32 // fix the options later
        {
            protected override void AddOptions()
            {
                AddOption("Single Player", 0);
                AddOption("Multiplayer", 1);
                AddOption("Shared", 2);
                AddOption("SP Shared", 3);
                AddOption("Mainmenu", 4);
            }
        }
        [Size(16)]
        Unused unknown2;
        Int fileStringsSize;
        Int fileStringsOffset;
        Int unknown4;
        Int string128Offset;
        Int stringsCount;
        Int stringTableSize;
        Int stringTableIndexOffset;
        Int stringTableOffset;
        [Size(36)]
        Unused unknown5;
        [Size(36)]
        String mapName;
        [Size(256)]
        String scenarioPath;
        Int unknown;
        Int fileTableCount;
        Int fileTableOffset;
        Int fileTableSize;
        Int filesIndexOffset;
        Int signature;
        [Size(1320)]
        Unused zero3;
        [Size(4)]
        String foot;

        #endregion

        public int IndexOffset
        {
            get { return indexHeaderOffset.Value; }
            set { indexHeaderOffset.Update(value); }
        }
        public int FileTableSize
        {
            get { return fileTableSize.Value; }
            set { fileTableSize.Update(value); }
        }
        public int FileTableOffset
        {
            get { return fileTableOffset.Value; }
            set { fileTableOffset.Update(value); }
        }
        public int FilesIndexOffset
        {
            get { return filesIndexOffset.Value; }
            set { filesIndexOffset.Update(value); }
        }
        public int MetaStart 
        { 
            get { return indexHeaderOffset.Value + indexHeaderSize.Value; }
        }
        public int StringsOffset
        {
            get { return stringTableOffset.Value; }
            set { stringTableOffset.Update(value); }
        }
        public int StringsCount
        {
            get { return stringsCount.Value; }
            set { stringsCount.Update(value); }
        }

        public string InternalName
        {
            get { return mapName.ToString(); }
        }

        public override void Read(EndianReader br)
        {
            br.BaseStream.Position = 0;
            base.Read(br);
        }

        public override void Write(EndianWriter bw)
        {
            bw.BaseStream.Position = 0;
            base.Write(bw);
        }
    }
}
