﻿
using System.IO;
using System.Windows.Forms;
using Microsoft.CSharp;
using System.CodeDom.Compiler;
using System;
using System.Collections;
using LazyIO;
using System.Collections.Generic;
using YeloDebug;
namespace HaloMap
{
    public class Map
    {
        #region properties

        public MapHeader Header
        {
            get;
            set;
        }
        public MapIndex Index
        {
            get;
            set;
        }
        public TagList Tags
        {
            get;
            set;
        }
        public StringList Strings
        {
            get;
            set;
        }
        //Hashtable MetasFromTagType
        //{
        //    get;
        //    set;
        //}
        IDictionary<string, Type> MetaTypesFromTagTypes { get; set; }
        EndianReader BR
        {
            get;
            set;
        }
        EndianWriter BW
        {
            get;
            set;
        }
        string FileName { get; set; }

        #endregion
        #region constructor

        public Map(string path)
        {
            FileName = path;
            Open();
            //MetasFromTagType = new Hashtable(85); // not more than 85 update types
            MetaTypesFromTagTypes = new Dictionary<string, Type>(85);
            Header = new MapHeader();
            Index = new MapIndex(Header);
            Strings = new StringList(Header);
            Tags = new TagList(Header, Index);
        }

        #endregion
        #region methods

        public Meta RetrieveMeta(Tag tag)
        {
            if (tag == null) { return null; }
            string format = "{0}\\plugins\\{1}.cs";
            string type = tag.Type;
            type = type.Replace("<", "").Replace(">", "");
            type = type.Trim();
            string filepath = string.Format(format, Application.StartupPath, type);
            if (File.Exists(filepath) == false) { return null; }
            //Meta hashed = (Meta)MetasFromTagType[filepath];
            //if (hashed != null)
            //{
            //    hashed.Update(tag, BR);
            //    return hashed;
            //}
            Type t = null;
            if( MetaTypesFromTagTypes.TryGetValue(filepath, out t) )
            {
                Meta meta = (Meta)Activator.CreateInstance(t);
                meta.Update(tag, BR);
                return meta;
            }
            else
            {
               t = CompileMeta(filepath);
            }
            if (t != null)
            {
                Meta meta = (Meta)Activator.CreateInstance(t);
                meta.Update(tag, BR);
                //MetasFromTagType.Add(filepath, meta);
                return meta;
            }
            return null;
        }
        private Type CompileMeta(string filepath)
        {
            CSharpCodeProvider myCodeProvider = new CSharpCodeProvider();
            CompilerParameters myCompilerParameters = new CompilerParameters();
            myCompilerParameters.ReferencedAssemblies.Add("System.dll");
            myCompilerParameters.ReferencedAssemblies.Add("System.Windows.Forms.dll");
            myCompilerParameters.ReferencedAssemblies.Add(Application.StartupPath + "\\HaloMap.dll");
            myCompilerParameters.ReferencedAssemblies.Add(Application.StartupPath + "\\LazyIO.dll");
            myCompilerParameters.GenerateExecutable = false;
            myCompilerParameters.GenerateInMemory = false;
            CompilerResults results = myCodeProvider.CompileAssemblyFromFile(myCompilerParameters, filepath);
            if (results.Errors.Count > 0)
            {
                foreach (CompilerError CompErr in results.Errors)
                {
                    MessageBox.Show("Line number " + CompErr.Line +
                        ", Error Number: " + CompErr.ErrorNumber +
                        ", '" + CompErr.ErrorText + ";" +
                        Environment.NewLine + Environment.NewLine);
                }
                return null;
            }  
            foreach (Type t in results.CompiledAssembly.GetTypes())
            {
                if (t.BaseType == typeof(Meta))
                {
                    return t;
                }
            }
            return null;;
        }
        public void SaveChanges(Meta meta)
        {
            meta.Write(BW);
        }
        void Open( )
        {
            FileStream fs = new FileStream(FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
            BR = new EndianReader(fs);
            BW = new EndianWriter(fs);
        }
        public void Load()
        {
            Header.Read(BR);
            Index.Read(BR);
            Strings.Read(BR);
            Tags.Read(BR);
            Close( );
        }
        public void Close()
        {
            if (BR != null)
            {
                BR.Close();
            }
            BR = null;
            BW = null;
        }
        public void Resign()
        {
            Open( );
            BR.BaseStream.Position = 2048;
            int left = (int)BR.BaseStream.Length - 2048;
            int times = left / 4;
            int sig = 0;
            while (times > 0)
            {
                sig ^= BR.ReadInt32();
                times--;
            }
            BW.BaseStream.Position = 720;
            BW.Write(sig);
            Close( );
        }
        public void Sync(Tag tag, Xbox xbox)
        {
            Open( );
            BR.BaseStream.Position = tag.MetaOffset;
            xbox.Setmem((uint)tag.RawMetaOffset, BR.ReadBytes(tag.MetaSize));
            Close( );
        }

        #endregion
    }
    
}
