﻿
using System.Collections.Generic;
using System.IO;
using System.Windows.Forms;
using LazyIO;
namespace HaloMap
{
    public abstract class BitMask<T> : MetaItemBase
    {
        #region -  properties   -

        public int Mask
        {
            get;
            private set;
        }
        public List<Bit> Bits
        {
            get;
            private set;
        }

        #endregion
        #region -  constructor  -

        public BitMask()
        {
            Bits = new List<Bit>();
            AddBits();
        }

        #endregion
        #region -    methods    -
        
        protected abstract void AddBits();
        protected void AddBit(string name, int index)
        {
            Bits.Add(new Bit(name, index));
        }
        public void Update(int value)
        {
            Mask = value;
        }

        #endregion
    }

    public class BitMask8 : BitMask<byte>
    {
        #region -  properties   -

        public sealed override int Size
        {
            get
            {
                return 1;
            }
        }

        #endregion
        #region -    methods    -

        protected override void AddBits()
        {
            // this will be invoked if user doesn't provide override
            Bits.Clear();
            int bits = 8;
            for (int i = 0; i < bits; i++)
            {
                AddBit(string.Format("Bit {0}", i), i);
            }
        }
        public sealed override void Read(EndianReader br)
        {
            Update((int)br.ReadByte());
        }
        public sealed override void Write(EndianWriter bw)
        {
            bw.Write((byte)Mask);
        }

        public sealed override string ToString()
        {
            return "BitMask8";
        }

        #endregion
    }

    public class BitMask16 : BitMask<short>
    {
        #region -  properties   -

        public sealed override int Size
        {
            get
            {
                return 2;
            }
        }

        #endregion        
        #region -    methods    -

        protected override void AddBits()
        {
            // this will be invoked if user doesn't provide override
            Bits.Clear();
            int bits = 16;
            for (int i = 0; i < bits; i++)
            {
                AddBit(string.Format("Bit {0}", i), i);
            }
        }
        public sealed override void Read(EndianReader br)
        {
            Update((int)br.ReadInt16());
        }
        public sealed override void Write(EndianWriter bw)
        {
            bw.Write((short)Mask);
        }
        public sealed override string ToString()
        {
            return "BitMask16";
        }

        #endregion
    }

    public class BitMask32 : BitMask<int>
    {
        #region -  properties   -

        #endregion
        #region -    methods    -

        protected override void AddBits()
        {
            // this will be invoked if user doesn't provide override
            Bits.Clear();
            int bits = 32;
            for (int i = 0; i < bits; i++)
            {
                AddBit(string.Format("Bit {0}", i), i);
            }
        }
        public sealed override void Read(EndianReader br)
        {
            Update(br.ReadInt32());
        }
        public sealed override void Write(EndianWriter bw)
        {
            bw.Write(Mask);
        }
        public sealed override string ToString()
        {
            return "BitMask32";
        }

        #endregion
    }

    public sealed class Bit
    {
        #region -  properties   -

        public string Name
        {
            get;
            private set;
        }
        public int Index
        {
            get;
            private set;
        }
        public int Flag
        {
            get { return 1 << Index; }
        }

        #endregion
        #region -  constructor  -

        public Bit(string name, int index)
        {
            Name = name;
            Index = index;
        }

        #endregion
        #region -    methods    -

        public override bool Equals(object obj)
        {
            if (obj is Bit)
            {
                Bit b = (Bit)obj;
                return b.Name.Equals(Name) && b.Index == Index;
            }
            return false;
        }
        public override int GetHashCode()
        {
            return ToString().GetHashCode();
        }
        public override string ToString()
        {
            return Name;
        }

        #endregion
    }
}
