﻿
using System.Collections.Generic;
using System.IO;
using System.Windows.Forms;
using LazyIO;
namespace HaloMap
{
    public abstract class Enum<T> : MetaItemBase
    {
        #region -   propertes   -

        public T Value
        {
            get;
            private set;
        }
        public List<Option<T>> Options
        {
            get;
            private set;
        }

        #endregion
        #region -  constructor  -

        public Enum()
        {
            Options = new List<Option<T>>();
            AddOptions();
        }

        #endregion
        #region -    methods    -

        protected virtual void AddOptions() { }
        protected void AddOption(string name, T value)
        {
            Options.Add(new Option<T>(name, value));
        }
        public void Update(T value)
        {
            Value = value;
        }

        #endregion
    }

    public class Enum8 : Enum<byte>
    {
        #region -  properties   -

        public sealed override int Size
        {
            get
            {
                return 1;
            }
        }

        #endregion
        #region -    methods    -

        public sealed override void Read(EndianReader br)
        {
            Update(br.ReadByte());
        }
        public sealed override void Write(EndianWriter bw)
        {
            bw.Write(Value);
        }
        public sealed override string ToString()
        {
            return "Enum8";
        }

        #endregion
    }
    public class Enum16 : Enum<short>
    {
        #region -  properties   -

        public sealed override int Size
        {
            get
            {
                return 2;
            }
        }

        #endregion
        #region -    methods    -

        public sealed override void Read(EndianReader br)
        {
            Update(br.ReadInt16());
        }
        public sealed override void Write(EndianWriter bw)
        {
            bw.Write(Value);
        }
        public sealed override string ToString()
        {
            return "Enum16";
        }

        #endregion
    }
    public class Enum32 : Enum<int>
    {
        #region -  properties   -

        public sealed override int Size
        {
            get
            {
                return 4;
            }
        }

        #endregion
        #region -    methods    -

        public sealed override void Read(EndianReader br)
        {
            Update(br.ReadInt32());
        }
        public sealed override void Write(EndianWriter bw)
        {
            bw.Write(Value);
        }
        public sealed override string ToString()
        {
            return "Enum32";
        }

        #endregion
    }

    public sealed class Option<T>
    {
        #region -  properties   -

        public string Name
        {
            get;
            private set;
        }
        public T Value
        {
            get;
            private set;
        }

        #endregion
        #region -  constructor  -

        public Option(string name, T value)
        {
            Name = name;
            Value = value;
        }

        #endregion
        #region -    methods    -

        public override sealed string ToString()
        {
            return Name;
        }

        #endregion
    }
}
