﻿using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Xml;
using LazyIO;

namespace HaloMap
{
    public class EntPlugin : Meta
    {
        public EntPlugin(string filepath)
        {
            XmlTextReader xtr = new XmlTextReader(new FileStream(filepath, FileMode.Open, FileAccess.Read));
            xtr.WhitespaceHandling = WhitespaceHandling.None;
            Items = new List<LazyIO.IMetaItem>( );
            while( xtr.Read( ) )
            {
                switch( xtr.NodeType )
                {
                    case XmlNodeType.Element:
                        {
                            switch( xtr.Name.ToLower( ) )
                            {
                                case "plugin":
                                    {
                                        Author = xtr.GetAttribute("author");
                                        Version = xtr.GetAttribute("version");
                                        TagClass = xtr.GetAttribute("class");
                                        break;
                                    }
                                case "byte":
                                    {
                                        LazyIO.Byte b = new LazyIO.Byte( );
                                        b.Name = xtr.GetAttribute("name");
                                        string visible = xtr.GetAttribute("visible");
                                        b.Visible = string.IsNullOrEmpty(visible) == false && visible.Equals("true");
                                        Items.Add(b);
                                        break;
                                    }
                                case "short":
                                    {
                                        LazyIO.Short s = new LazyIO.Short( );
                                        s.Name = xtr.GetAttribute("name");
                                        string visible = xtr.GetAttribute("visible");
                                        s.Visible = string.IsNullOrEmpty(visible) == false && visible.Equals("true");
                                        Items.Add(s);
                                        break;
                                    }
                                case "ushort":
                                    {
                                        LazyIO.UShort us = new LazyIO.UShort( );
                                        us.Name = xtr.GetAttribute("name");
                                        string visible = xtr.GetAttribute("visible");
                                        us.Visible = string.IsNullOrEmpty(visible) == false && visible.Equals("true");
                                        Items.Add(us);
                                        break;
                                    }
                                case "int":
                                    {
                                        LazyIO.Int i = new LazyIO.Int( );
                                        i.Name = xtr.GetAttribute("name");
                                        string visible = xtr.GetAttribute("visible");
                                        i.Visible = string.IsNullOrEmpty(visible) == false && visible.Equals("true");
                                        Items.Add(i);
                                        break;
                                    }
                                case "uint":
                                    {
                                        LazyIO.UInt ui = new LazyIO.UInt( );
                                        ui.Name = xtr.GetAttribute("name");
                                        string visible = xtr.GetAttribute("visible");
                                        ui.Visible = string.IsNullOrEmpty(visible) == false && visible.Equals("true");
                                        Items.Add(ui);
                                        break;
                                    }
                                case "float":
                                    {
                                        LazyIO.Float f = new LazyIO.Float( );
                                        f.Name = xtr.GetAttribute("name");
                                        string visible = xtr.GetAttribute("visible");
                                        f.Visible = string.IsNullOrEmpty(visible) == false && visible.Equals("true");
                                        Items.Add(f);
                                        break;
                                    }
                                case "bitmask8":
                                    {
                                        BitMask8 bm = new BitMask8( );
                                        bm.Name = xtr.GetAttribute("name");
                                        string visible = xtr.GetAttribute("visible");
                                        bm.Visible = string.IsNullOrEmpty(visible) == false && visible.Equals("true");
                                        Items.Add(bm);
                                        break;
                                    }
                                case "bitmask16":
                                    {
                                        BitMask16 bm = new BitMask16( );
                                        bm.Name = xtr.GetAttribute("name");
                                        string visible = xtr.GetAttribute("visible");
                                        bm.Visible = string.IsNullOrEmpty(visible) == false && visible.Equals("true");
                                        // code for getting bits added
                                        Items.Add(bm);
                                        break;
                                    }
                                case "bitmask":
                                case "bitmask32":                                
                                    {
                                        BitMask32 bm = new BitMask32();
                                        bm.Name = xtr.GetAttribute("name");
                                        string visible = xtr.GetAttribute("visible");
                                        bm.Visible = string.IsNullOrEmpty(visible) == false && visible.Equals("true");
                                        // code for getting bits added
                                        Items.Add(bm);
                                        break;
                                    }
                                case "enum8":
                                    {
                                        Enum8 en = new Enum8( );
                                        en.Name = xtr.GetAttribute("name");
                                        string visible = xtr.GetAttribute("visible");
                                        en.Visible = string.IsNullOrEmpty(visible) == false && visible.Equals("true");
                                        // code for getting options here
                                        Items.Add(en);
                                        break;
                                    }
                                case "enum16":
                                    {
                                        Enum16 en = new Enum16( );
                                        en.Name = xtr.GetAttribute("name");
                                        string visible = xtr.GetAttribute("visible");
                                        en.Visible = string.IsNullOrEmpty(visible) == false && visible.Equals("true");
                                        // code for getting options here
                                        Items.Add(en);
                                        break;
                                    }
                                case "enum":
                                case "enum32":
                                    {
                                        Enum32 en = new Enum32( );
                                        en.Name = xtr.GetAttribute("name");
                                        string visible = xtr.GetAttribute("visible");
                                        en.Visible = string.IsNullOrEmpty(visible) == false && visible.Equals("true");
                                        // code for getting options here
                                        Items.Add(en);
                                        break;
                                    }
                                case "ident":
                                    {
                                        // code to deal with dependencies
                                        break;
                                    }
                                case "stringid":
                                    {
                                        StringID sid = new StringID( );
                                        sid.Name = xtr.GetAttribute("name");
                                        string visible = xtr.GetAttribute("visible");
                                        sid.Visible = string.IsNullOrEmpty(visible) == false && visible.Equals("true");
                                        Items.Add(sid);
                                        break;
                                    }
                                case "unused":
                                    {
                                        int size = 4;
                                        try { size = int.Parse(xtr.GetAttribute("size")); }
                                        catch { }
                                        Unused u = new Unused(size);                                        
                                        u.Name = xtr.GetAttribute("offset");
                                        Items.Add(u);
                                        break;
                                    }
                                case "undefined":
                                    {
                                        Unknown u = new Unknown( );
                                        u.Name = xtr.GetAttribute("name");
                                        string visible = xtr.GetAttribute("visible");
                                        u.Visible = string.IsNullOrEmpty(visible) == false && visible.Equals("true");
                                        Items.Add(u);
                                        break;
                                    }
                                case "struct":
                                    {
                                        // would need code generation for this for the reflexive type
                                        break;
                                    }
                            }
                            break;
                        }
                }
            }
        }
    }
}
