﻿using System.Drawing;
using System.Windows.Forms;

namespace HaloMap
{
    public partial class BitMaskViewer<T> : MetaItemViewer
    {
        #region -    fields     -

        int padding;
        BitMask<T> _bitmask;

        #endregion
        #region -  constructor  -

        public BitMaskViewer()
        {
            InitializeComponent();
        }
        public BitMaskViewer(BitMask<T> bitmask)
            :this()
        {
            _bitmask = bitmask;
            gb.Text = MetaItemBase.GetNameLabel(bitmask);
            padding = GetPadding();
            MakeControls();
            Size = PreferredSize;
            gb.Size = gb.PreferredSize;
            Dock = DockStyle.Top;
        }

        
        #endregion
        #region -    methods    -

        private int GetPadding()
        {
            int largest = 50;
            if (_bitmask.Bits != null && _bitmask.Bits.Count > 0)
            {
                foreach (Bit b in _bitmask.Bits)
                {
                    checkBox1.Text = b.Name;
                    if (checkBox1.PreferredSize.Width > largest)
                    {
                        largest = checkBox1.PreferredSize.Width + 5;
                    }
                }
            }
            return largest;
        }
        private void MakeControls()
        {
            gb.Controls.Clear();
            for (int i = 0; i < _bitmask.Bits.Count; i++)
            {
                CheckBox temp = new CheckBox();
                temp.AutoSize = true;
                temp.Size = new System.Drawing.Size(80, 17);
                temp.Text = _bitmask.Bits[i].Name;
                temp.Text.PadLeft(padding);
                temp.UseVisualStyleBackColor = true;
                temp.CheckedChanged += new System.EventHandler(CheckBox_CheckedChanged);
                temp.Tag = _bitmask.Bits[i];
                gb.Controls.Add(temp);
            }
            int down = 30;
            int over = 6;
            int CounterForPlacementKeeping = 0;
            foreach (Control c in gb.Controls)
            {
                c.Location = new Point(over, down);
                over += padding;
                if ((++CounterForPlacementKeeping) % 4 == 0)
                {
                    down += 30;
                    over = 6;
                }
            }
            gb.Size = gb.PreferredSize;
            Size = new System.Drawing.Size(PreferredSize.Width, PreferredSize.Height + 3);
        }
        private void SetChecks()
        {
            foreach (Control c in gb.Controls)
            {
                if (c.GetType() == typeof(CheckBox))
                {
                    CheckBox temp = (CheckBox)c;
                    Bit b = temp.Tag as Bit;
                    if (b != null)
                    {
                        int result = _bitmask.Mask & b.Flag;
                        temp.Checked = result != 0;
                    }
                }
            }
        }
        private void CheckBox_CheckedChanged(object sender, System.EventArgs e)
        {
            CheckBox cb = sender as CheckBox;
            if (cb != null)
            {
                Bit b = cb.Tag as Bit;
                if (b != null)
                {
                    if (cb.Checked)
                    {
                        _bitmask.Update(_bitmask.Mask | b.Flag);
                    }
                    else
                    {
                        _bitmask.Update(_bitmask.Mask & ~b.Flag);
                    }
                }
            }
        }
        public override void UpdateControl()
        {
            BeginUpdate();
            SetChecks();
            EndUpdate();
        }

        #endregion
    }
}
