﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;

namespace HaloMap
{
    public partial class EnumViewer<T> : MetaItemViewer
    {
        #region -  properties   -

        Enum<T> value
        {
            get;
            set;
        }

        #endregion
        #region -  constructor  -

        public EnumViewer()
        {
            InitializeComponent();
        }
        public EnumViewer(Enum<T> e)
            : this()
        {
            value = e;
            string type = "Enum";
            Type t = e.GetType();
            if (t.IsSubclassOf(typeof(Enum8))) { type = "Enum8"; }
            else if (t.IsSubclassOf(typeof(Enum16))) { type = "Enum16"; }
            else if (t.IsSubclassOf(typeof(Enum32))) { type = "Enum32"; }
            lblType.Text = type;
            lblNameAndComments.Text = MetaItemBase.GetNameLabel(e);
            Dock = DockStyle.Top;
        }

        #endregion
        #region -    methods    -

        private void SetUp()
        {
            cmbOptions.Items.Clear();
            List<Option<T>> options = value.Options;
            cmbOptions.Items.AddRange(options.ToArray());
            bool found = false;
            for (int i = 0; i < options.Count; i++)
            {
                // if it's in the list set the combobox to point
                // to the index it's at
                if (options[i].Value.Equals(value.Value))
                {
                    cmbOptions.SelectedIndex = i;
                    found = true;
                    break;
                }
            }
            if (found == false)
            {
                cmbOptions.Items.Add(new Option<T>(value.Value.ToString(), value.Value));
                cmbOptions.SelectedIndex = cmbOptions.Items.Count - 1;
            }
            SetSize();
        }
        private void SetSize()
        {
            cmbOptions.Width = (cmbOptions.Text.Length * 5) + 50;
            int labelX = cmbOptions.Location.X + cmbOptions.Width + 6;
            lblNameAndComments.Location = new Point(labelX, lblNameAndComments.Location.Y);
            Size = PreferredSize;
        }
        private void cmbOptions_SelectedIndexChanged(object sender, EventArgs e)
        {
            Option<T> o = cmbOptions.SelectedItem as Option<T>;
            if (o != null)
            {
                value.Update(o.Value);
            }
            SetSize();

        }
        public override void UpdateControl()
        {
            BeginUpdate();
            SetUp();
            EndUpdate();
        }

        #endregion
    }
}
