﻿using System.Windows.Forms;
using System.Drawing;

namespace HaloMap
{
    public partial class IdentViewer : MetaItemViewer
    {
        #region properties

        public static TagList Tags
        {
            get;
            set;
        }

        Ident ID
        {
            get;
            set;
        }

        #endregion
        #region constructor

        public IdentViewer()
        {
            InitializeComponent();
            Dock = DockStyle.Top;
        }

        public IdentViewer(Ident id)
            :this()
        {
            ID = id;
            lblType.Text = id.GetType() == typeof(Dependency) ? "Dependency" : "Ident";
            lblNameAndComments.Text = MetaItemBase.GetNameLabel(id);
            SetUp();
            cmbType.SelectedIndexChanged += new System.EventHandler(Type_Changed);
            cmbName.SelectedIndexChanged += new System.EventHandler(Name_Changed);

        }

        #endregion
        #region methods

        void Name_Changed(object sender, System.EventArgs e)
        {
            Tag t = Tags.FindTag(cmbType.Text, cmbName.Text);
            if (t != null)
            {
                ID.Update(t.Ident);
            }
            if(cmbName.Text.Equals("nulled"))
            {
                ID.Update(-1);
            }
            SetSize();
        }
        void Type_Changed(object sender, System.EventArgs e)
        {
            if (cmbType.Text.Equals("nulled"))
            {
                cmbName.Items.Clear();
                cmbName.Text = "nulled";
                SetSize();
                ID.Update(-1);
            }
            else
            {
                cmbName.Items.Clear();
                cmbName.Sorted = true;
                string[] tags = Tags.GetAllTagsOfType(cmbType.Text);
                if (tags != null)
                {
                    cmbName.Items.AddRange(Tags.GetAllTagsOfType(cmbType.Text));
                }
                cmbName.Sorted = false;
                cmbName.Items.Add("nulled");
            }
        }
        void SetUp()
        {
            cmbType.Sorted = true;
            cmbType.Items.Clear();
            cmbType.Items.AddRange(Tags.TagTypes.ToArray());
            cmbType.Sorted = false;
            cmbType.Items.Add("nulled");
            Tag t = null;
            if (ID.Value != -1 && (t = Tags.FindTag(ID.Value)) != null)
            {
                cmbType.Text = t.Type;
                cmbName.Text = t.Name;
                cmbName.Items.AddRange(Tags.GetAllTagsOfType(t.Type));
            }
            else
            {
                cmbType.Text = "nulled";
                cmbName.Text = "nulled";
            }
            SetSize();
        }
        void SetSize()
        {
            cmbName.Width = (cmbName.Text.Length * 5) + 50;
            int labelX = cmbName.Location.X + cmbName.Width + 6;
            lblNameAndComments.Location = new Point(labelX, lblNameAndComments.Location.Y);
            Size = PreferredSize;
        }
        public override void UpdateControl()
        {
            BeginUpdate();
            SetUp();
            EndUpdate();
        }

        #endregion
    }
}
