﻿using System;
using System.Drawing;
using System.Windows.Forms;
using LazyIO;

namespace HaloMap
{
    public partial class NumberViewer<T> : MetaItemViewer
    {
        #region -    fields     -

        Number<T> num;
        string type;
        Color normalColor;
        Color errorColor;

        #endregion
        #region -  constructor  -

        public NumberViewer()
        {
            InitializeComponent();
        }
        public NumberViewer(Number<T> number)
            :this()
        {
            num = number;
            type = number.GetStringType();
            normalColor = txtNum.BackColor;
            errorColor = Color.Yellow;
            lblType.Text = type;
            lblNameAndComments.Text = MetaItemBase.GetNameLabel(num);
            int spacing = 6;
            int x = lblType.Location.X + lblType.Size.Width + spacing;
            txtNum.Location = new Point(x, txtNum.Location.Y);
            x = txtNum.Location.X + txtNum.Size.Width + spacing;
            lblNameAndComments.Location = new Point(x, lblType.Location.Y);
            Size = PreferredSize;
            AutoSize = false;
            Dock = DockStyle.Top;
        }

        #endregion
        #region -    methods    -

        public void SetUp()
        {           
            txtNum.Text = num.Value.ToString();           
        }
        private void SetError()
        {
            // if they've already been notified just return
            if (txtNum.BackColor == errorColor) { return; }
            txtNum.BackColor = errorColor;
            string message = string.Format("{0}: You must enter a valid {1}", num.Name, type);
            MessageBox.Show(message, "Invalid Input");
            txtNum.SelectAll();
        }
        private void Text_Changed(object sender, System.EventArgs e)
        {
            string value = txtNum.Text.Trim();
            if (string.IsNullOrEmpty(value)){return;}
            if(value.Equals("-")){return;}
            Type t = typeof(T);
            if (t == typeof(byte))
            {
                byte temp = 0;
                if (byte.TryParse(value, out temp))
                {
                    Byte b = num as Byte;
                    b.Update(temp);
                    txtNum.BackColor = normalColor;
                }
                else { SetError(); }
            }
            else if (t == typeof(ushort))
            {
                ushort temp = 0;
                if (ushort.TryParse(value, out temp))
                {
                    UShort us = num as UShort;
                    us.Update(temp);
                    txtNum.BackColor = normalColor;
                }
                else { SetError(); }
            }
            else if (t == typeof(short))
            {
                short temp = 0;
                if (short.TryParse(value, out temp))
                {
                    Short s = num as Short;
                    s.Update(temp);
                    txtNum.BackColor = normalColor;
                }
                else { SetError(); }
            }
            else if (t == typeof(uint))
            {
                uint temp = 0;
                if (uint.TryParse(value, out temp))
                {
                    UInt ui = num as UInt;
                    ui.Update(temp);
                    txtNum.BackColor = normalColor;
                }
                else { SetError(); }
            }
            else if (t == typeof(int))
            {
                int temp = 0;
                if (int.TryParse(value, out temp))
                {
                    Int i = num as Int;
                    i.Update(temp);
                    txtNum.BackColor = normalColor;
                }
                else { SetError(); }
            }
            else if (t == typeof(float))
            {
                float temp = 0;
                if (float.TryParse(value, out temp))
                {
                    Float s = num as Float;
                    s.Update(temp);
                    txtNum.BackColor = normalColor;
                }
                else { SetError(); }
            }
        }
        private void Key_Down(object sender, KeyEventArgs e)
        {
            if (e.Control && e.KeyCode == Keys.A)
            {
                txtNum.SelectAll();
            }
        }
        private void Key_Press(object sender, KeyPressEventArgs e)
        {
            // 45 =  minus
            // 46 = decimal
            Type t = typeof(T);
            bool decimalAllowed = t == typeof(float) || t == typeof(double);
            bool minusAllowed = (t == typeof(byte) || t == typeof(ushort) ||
                            t == typeof(uint) || t == typeof(ulong)) == false;
            char c = e.KeyChar;
            if (char.IsControl(c) || char.IsNumber(c))
            {
                e.Handled = false;
            }
            else if (c == 45 && minusAllowed)
            {
                e.Handled = false;
            }
            else if (c == 46 && decimalAllowed)
            {
                e.Handled = false;
            }
            else
            {
                e.Handled = true;
            }
        }
        public override void UpdateControl()
        {
            BeginUpdate();
            SetUp();
            EndUpdate();
        }

        #endregion
    }
}
