﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.IO;
using System.Windows.Forms;
using LazyIO;

namespace HaloMap
{
    public interface IMagicReadWrite
    {
        void Read(EndianReader br, int magic);
    }

    public sealed class Reflexive<T> : List<T>, IMetaItem, IMagicReadWrite where T : Chunk
    {
        #region -  properties   -

        public int ChunkCount
        {
            get;
            set;
        }
        public int Offset
        {
            get;
            set;
        }
        public int Translation
        {
            get { return Offset - Magic; }
        }
        public int Magic
        {
            get;
            set;
        }
        

        public string Name
        {
            get;
            set;
        }
        public string Description
        {
            get;
            set;
        }
        public bool Visible
        {
            get;
            set;
        }
        public int Size
        {
            get
            {
                return 8 + GetSize();
            }
        }

        const int ChunksAllowed = 1000000;

        #endregion

        #region -    methods    -

        int GetSize()
        {
            int size = 0;
            foreach (T chunk in this)
            {
                size += chunk.Size;
            }
            return size;
        }
        public void Read(EndianReader br)
        {
            ChunkCount = br.ReadInt32( );
            Offset = br.ReadInt32();
            if( ChunkCount > ChunksAllowed ) { throw new Exception(string.Format("Reflexive Chunk Size Over {0}..  Check your plugin", ChunksAllowed)); }
            if (ChunkCount > 0)
            {
                long temp = br.BaseStream.Position;
                // remember we're reading from just the Tags meta not the whole map
                br.BaseStream.Position = Translation;
                Clear();
                for (int i = 0; i < ChunkCount; i++)
                {
                    T chunk = (T)Activator.CreateInstance(typeof(T));
                    chunk.Read(br, Magic);
                    Add(chunk);
                }
                br.BaseStream.Position = temp;
            }
        }
        public void Read(EndianReader br, int magic)
        {
            Magic = magic;
            Read(br);
        }
        public void Write(EndianWriter bw)
        {
            bw.Write(ChunkCount);
            bw.Write(Offset);
            if( ChunkCount > 0 )
            {
                long temp = bw.BaseStream.Position;
                bw.BaseStream.Position = Translation;
                foreach( T chunk in this )
                {
                    chunk.Write(bw);
                }
                bw.BaseStream.Position = temp;
            }
        }
        public override string ToString( )
        {
            if( string.IsNullOrEmpty(Description) == false )
            {
                return Name + " {" + Description + "}";
            }
            return Name;
        }

        #endregion
    }
}
