﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;
using LazyIO;

namespace HaloMap
{
    public partial class EnumViewer<T> : UserControl
    {
        #region -  properties   -

        Enum<T> Value
        {
            get;
            set;
        }

        public string ObjectName
        {
            get { return Value != null ? Value.Name : "null"; }
        }

        #endregion

        #region -  constructor  -

        public EnumViewer()
        {
            InitializeComponent();
            Dock = DockStyle.Top;
        }

        #endregion
        
        #region -    methods    -

        public void SetObject(Enum<T> e)
        {
            Value = e;
            string type = "Enum";
            Type t = e.GetType( );
            if( t.IsSubclassOf(typeof(Enum8)) ) { type = "Enum8"; }
            else if( t.IsSubclassOf(typeof(Enum16)) ) { type = "Enum16"; }
            else if( t.IsSubclassOf(typeof(Enum32)) ) { type = "Enum32"; }
            lblType.Text = type;
            lblNameAndComments.Text = MetaItemBase.GetNameLabel(e);
            cmbOptions.Items.Clear( );
            cmbOptions.Items.AddRange(Value.Options.ToArray( ));
            int idx = -1;
            for( int i = 0; i < e.Options.Count; i++ )
            {
                if( e.Options[i].Value.Equals(e.Value) )
                {
                    idx = i;
                }
            }
            cmbOptions.SelectedIndexChanged -= new EventHandler(cmbOptions_SelectedIndexChanged);
            cmbOptions.SelectedIndex = idx;
            if( idx == -1 ) { cmbOptions.Items.Add(e.Value.ToString( )); cmbOptions.SelectedIndex = cmbOptions.Items.Count - 1; }
            SetSize( );
            cmbOptions.SelectedIndexChanged += new EventHandler(cmbOptions_SelectedIndexChanged);

        }
        private void SetSize( )
        {
            Graphics g = Graphics.FromHwnd(cmbOptions.Handle);
            cmbOptions.Width = (int)g.MeasureString(cmbOptions.Text, cmbOptions.Font).Width + 50;
            g.Dispose( );
            Size = PreferredSize;
            
        }
        private void cmbOptions_SelectedIndexChanged(object sender, EventArgs e)
        {
            Option<T> o = cmbOptions.SelectedItem as Option<T>;
            if (o != null)
            {
                Value.Update(o.Value);
                SetSize( );
            }
        }

        #endregion
    }
}
