﻿using System.Windows.Forms;
using System.Drawing;
using LazyIO;

namespace HaloMap
{
    public partial class IdentViewer : UserControl
    {
        #region -  properties   -

        TagList Tags
        {
            get;
            set;
        }

        Ident Value
        {
            get;
            set;
        }

        public string ObjectName
        {
            get{return Value != null ? Value.Name : "null";}
        }

        #endregion

        #region -  constructor  -

        public IdentViewer()
        {
            InitializeComponent();
        }

        public IdentViewer(TagList tags)
        {
            InitializeComponent( );
            Tags = tags;
            cmbType.SelectedIndexChanged += new System.EventHandler(Type_Changed);
            cmbName.SelectedIndexChanged += new System.EventHandler(Name_Changed);
            cmbType.DropDown += new System.EventHandler(cmbType_DropDown);
            Dock = DockStyle.Top;
        }

        

        #endregion

        #region -    methods    -

        public void SetObject(Ident value)
        {
            cmbType.SelectedIndexChanged -= new System.EventHandler(Type_Changed);
            cmbName.SelectedIndexChanged -= new System.EventHandler(Name_Changed);
            Value = value;
            lblType.Text = value is Dependency ? "Dependency" : "Ident";
            lblNameAndComments.Text = MetaItemBase.GetNameLabel(value);
            cmbType.SelectedIndexChanged += new System.EventHandler(Type_Changed);
            cmbName.SelectedIndexChanged += new System.EventHandler(Name_Changed);

            Tag t = null;
            if( Value.Value != -1 && ( t = Tags.FindTag(Value.Value) ) != null )
            {
                // set the type and name
                // add all the tags of the type to the names box
                cmbType.Text = t.Type;
                cmbName.Text = t.Name;
                cmbName.Items.AddRange(Tags.GetAllTagsOfType(t.Type));
            }
            else
            {
                cmbType.Text = "nulled";
                cmbName.Text = "nulled";
            }
            SetSize( );
            // SetSize highlights the name combo box
            // this fixes it
            cmbName.SelectedText = cmbName.SelectedText;
        }
        void Type_Changed(object sender, System.EventArgs e)
        {
            if (cmbType.Text.Equals("nulled"))
            {
                cmbName.Items.Clear();
                cmbName.Text = "nulled";
                SetSize();
                Value.Update(-1);
            }
            else
            {
                cmbName.Items.Clear();
                cmbName.Sorted = true;
                string[] tags = Tags.GetAllTagsOfType(cmbType.Text);
                if (tags != null)
                {
                    cmbName.Items.AddRange(tags);
                }
                cmbName.Sorted = false;
                cmbName.Items.Add("nulled");
                cmbName.SelectedIndex = 0;
            }
        }
        void Name_Changed(object sender, System.EventArgs e)
        {
            Tag t = Tags.FindTag(cmbType.Text, cmbName.Text);
            if( t != null )
            {
                Value.Update(t.Ident);
            }
            if( cmbName.Text.Equals("nulled") )
            {
                cmbType.Text = "nulled";
                Value.Update(-1);
            }
            SetSize( );
        }
        void cmbType_DropDown(object sender, System.EventArgs e)
        {
            cmbType.Sorted = true;
            cmbType.Items.AddRange(Tags.TagTypes.ToArray( ));
            cmbType.Sorted = false;
            cmbType.Items.Add("nulled");
            // should only happen the first time dropping down
            cmbType.DropDown -= new System.EventHandler(cmbType_DropDown);
        }
        void SetSize()
        {
            Graphics g = Graphics.FromHwnd(Handle);
            cmbName.Width = (int)g.MeasureString(cmbName.Text, Font).Width + 40;
            g.Dispose( );
            Size = PreferredSize;
        }

        #endregion
    }
}
