﻿using System;
using System.Drawing;
using System.Windows.Forms;
using LazyIO;

namespace HaloMap
{
    public partial class NumberViewer<T> : UserControl
    {
        #region -    fields     -

        Number<T> Value { get; set; }
        public string ObjectName
        {
            get { return Value != null ? Value.Name : "null"; }
        }
        string type { get; set; }
        Color NormalColor { get; set; }
        Color ErrorColor { get; set; }

        #endregion

        #region -  constructor  -

        public NumberViewer()
        {
            InitializeComponent();
            NormalColor = txtNum.BackColor;
            ErrorColor = Color.Yellow;
            type = "";
            Dock = DockStyle.Top;
        }

        #endregion

        #region -    methods    -

        public void SetObject(Number<T> number)
        {
            txtNum.TextChanged -= new EventHandler(Text_Changed);
            Value = number;
            type = number.GetStringType( );
            lblType.Text = type;
            lblNameAndComments.Text = MetaItemBase.GetNameLabel(Value);
            txtNum.Text = number.ToString( );
            txtNum.BackColor = NormalColor;
            PerformLayout( );
            txtNum.TextChanged += new EventHandler(Text_Changed);
            Size = PreferredSize;
            //// layout code
            //int spacing = 6;
            //int x = lblType.Location.X + lblType.Size.Width + spacing;
            //txtNum.Location = new Point(x, txtNum.Location.Y);
            //x = txtNum.Location.X + txtNum.Size.Width + spacing;
            //lblNameAndComments.Location = new Point(x, lblType.Location.Y);
            //Size = PreferredSize;           
        }
        private void SetError()
        {
            // if they've already been notified just return
            if (txtNum.BackColor == ErrorColor) { return; }
            txtNum.BackColor = ErrorColor;
            string message = string.Format("{0}: You must enter a valid {1}", Value.Name, type);
            MessageBox.Show(message, "Invalid Input");
            txtNum.SelectAll();
        }
        private void Text_Changed(object sender, System.EventArgs e)
        {

            string value = txtNum.Text.Trim( );
            if( string.IsNullOrEmpty(value) ) { return; }
            if( value.Equals("-") ) { return; }
            bool updated = Value.Update(value);
            if( Value.Update(value) )
            {
                txtNum.BackColor = NormalColor;
            }
            else
            {
                SetError( );
            }

            //// old update code
            //string value = txtNum.Text.Trim();
            //if (string.IsNullOrEmpty(value)){return;}
            //if(value.Equals("-")){return;}
            //Type t = typeof(T);
            //if (t == typeof(byte))
            //{
            //    byte temp = 0;
            //    if (byte.TryParse(value, out temp))
            //    {
            //        LazyIO.Byte b = Value as LazyIO.Byte;
            //        b.Update(temp);
            //        txtNum.BackColor = NormalColor;
            //    }
            //    else { SetError(); }
            //}
            //else if (t == typeof(ushort))
            //{
            //    ushort temp = 0;
            //    if (ushort.TryParse(value, out temp))
            //    {
            //        UShort us = Value as UShort;
            //        us.Update(temp);
            //        txtNum.BackColor = NormalColor;
            //    }
            //    else { SetError(); }
            //}
            //else if (t == typeof(short))
            //{
            //    short temp = 0;
            //    if (short.TryParse(value, out temp))
            //    {
            //        Short s = Value as Short;
            //        s.Update(temp);
            //        txtNum.BackColor = NormalColor;
            //    }
            //    else { SetError(); }
            //}
            //else if (t == typeof(uint))
            //{
            //    uint temp = 0;
            //    if (uint.TryParse(value, out temp))
            //    {
            //        UInt ui = Value as UInt;
            //        ui.Update(temp);
            //        txtNum.BackColor = NormalColor;
            //    }
            //    else { SetError(); }
            //}
            //else if (t == typeof(int))
            //{
            //    int temp = 0;
            //    if (int.TryParse(value, out temp))
            //    {
            //        Int i = Value as Int;
            //        i.Update(temp);
            //        txtNum.BackColor = NormalColor;
            //    }
            //    else { SetError(); }
            //}
            //else if (t == typeof(float))
            //{
            //    float temp = 0;
            //    if (float.TryParse(value, out temp))
            //    {
            //        Float s = Value as Float;
            //        s.Update(temp);
            //        txtNum.BackColor = NormalColor;
            //    }
            //    else { SetError(); }
            //}
        }
        private void Key_Down(object sender, KeyEventArgs e)
        {
            if (e.Control && e.KeyCode == Keys.A)
            {
                txtNum.SelectAll();
            }
        }
        private void Key_Press(object sender, KeyPressEventArgs e)
        {
            // 45 =  minus
            // 46 = decimal
            Type t = typeof(T);
            bool decimalAllowed = t == typeof(float) || t == typeof(double);
            bool minusAllowed = (t == typeof(byte) || t == typeof(ushort) ||
                            t == typeof(uint) || t == typeof(ulong)) == false;
            char c = e.KeyChar;
            if (char.IsControl(c) || char.IsNumber(c))
            {
                e.Handled = false;
            }
            else if (c == 45 && minusAllowed)
            {
                e.Handled = false;
            }
            else if (c == 46 && decimalAllowed)
            {
                e.Handled = false;
            }
            else
            {
                e.Handled = true;
            }
        }

        #endregion
    }
}
