﻿using System;
using System.Collections;
using System.Windows.Forms;
using LazyIO;

namespace HaloMap
{
    public partial class ReflexiveViewer : UserControl
    {
        #region -  properties   -

        bool Reload { get; set; }
        IList reflexive { get; set; }
        public string ObjectName
        {
            get { return reflexive != null ? ( (IMetaItem)reflexive ).Name : "null"; }
        }

        TagList Tags { get; set; }
        StringList Strings { get; set; }

        ViewType viewing;
        public ViewType Viewing
        {
            get { return viewing; }
            set { ChangeViewType(value); }
        }

        #endregion

        #region -  constructor  -

        public ReflexiveViewer( )
        {
            InitializeComponent( );
        }

        public ReflexiveViewer(TagList tags, StringList strings)
        {
            InitializeComponent( );
            Tags = tags;
            Strings = strings;
            viewing = ViewType.All;
            Dock = DockStyle.Top;
        }

        #endregion

        #region -    methods    -

        void LoadChunk( )
        {
            if( cmbChunks.SelectedIndex == -1 )
                return;
            if( reflexive.Count == 0 )
                return;

            Chunk c = (Chunk)reflexive[cmbChunks.SelectedIndex];
            if( Reload )
            {
                c.RepopulatePanel(panel);
            }
            else
            {
                c.PopulatePanel(panel, Viewing, Tags, Strings);
            }
            panel.Dock = DockStyle.None;
            panel.Size = panel.PreferredSize;
            Size = PreferredSize;
            panel.Dock = DockStyle.Fill;
            Reload = true;
        }

        public void SetObject(IList value)
        {
            reflexive = value;
            IMetaItem mi = (IMetaItem)value;
            string nameAndDescription = mi.Name;
            nameAndDescription += string.IsNullOrEmpty(mi.Description) ? "" : " (" + mi.Description + ")";
            // if it's greater than zero this should be enabled
            if( value.Count == 0 )
            {
                cmbChunks.Items.Clear( );
                string text = string.Format("0 : {0}", nameAndDescription);
                cmbChunks.Items.Add(text);
                cmbChunks.SelectedIndex = 0;
            }
            else
            {
                cmbChunks.BeginUpdate( );
                cmbChunks.Items.Clear( );
                for( int i = 0; i < value.Count; i++ )
                {
                    cmbChunks.Items.Add(string.Format("{0} : {1}", i, nameAndDescription));
                }
                cmbChunks.EndUpdate( );
                cmbChunks.SelectedIndex = 0; // fire the (re)populate method
            }
            Enabled = value.Count > 0;
        }

        void ChangeViewType(ViewType update)
        {
            if( viewing == update )
                return;
            foreach( Control c in this.panel.Controls )
            {
                // have it update each of the nest reflexive controls
                // which should mean they update their values
                if( c is ReflexiveViewer )
                {
                    ReflexiveViewer viewer = (ReflexiveViewer)c;
                    viewer.Viewing = update;
                }
            }
            Reload = false; // we want it to completely reload all the controls instead of just restting them
            LoadChunk( );
        }

        void SelectedChunkChanged(object sender, EventArgs e)
        {
            LoadChunk( );
        }

        #endregion
    }
}
