﻿using System.Windows.Forms;
using System.Drawing;
using LazyIO;

namespace HaloMap
{
    public partial class SIDViewer : UserControl
    {
        #region properties

        StringList Strings
        {
            get;
            set;
        }

        StringID Value
        {
            get;
            set;
        }

        public string ObjectName
        {
            get { return Value != null ? Value.Name : "null"; }
        }

        #endregion

        #region constructor

        public SIDViewer()
        {
            InitializeComponent();
        }

        public SIDViewer(StringList strings)
        {
            InitializeComponent( );
            cmbSID.DropDown += new System.EventHandler(cmbSID_DropDown);
            cmbSID.SelectedIndexChanged += new System.EventHandler(Selected_StringID_Changed);
            Strings = strings;
            Dock = DockStyle.Top;
        }

        #endregion

        #region methods
        
        public void SetObject(StringID sid)
        {
            Value = sid;
            lblNameAndComments.Text = MetaItemBase.GetNameLabel(sid);
            if( sid.Index >= 0 && sid.Index < Strings.Count )
            {
                cmbSID.Text = Strings[sid.Index].Value;
            }
            else
            {
                cmbSID.Text = "";
            }
            cmbSID.Size = cmbSID.PreferredSize;
            SetSize( );
        }
        void SetSize()
        {
            Graphics g = Graphics.FromHwnd(Handle);
            cmbSID.Width = (int)g.MeasureString(cmbSID.Text, Font).Width + 40;
            g.Dispose( );
            Size = PreferredSize;
        }
        void Selected_StringID_Changed(object sender, System.EventArgs e)
        {
            int idx = cmbSID.SelectedIndex;
            if (idx < Strings.Count && idx >= 0)
            {
                HaloString hs = Strings[idx];
                Value.Update(hs.Index, (byte)hs.Value.Length);
            }
            SetSize( );
        }
        void cmbSID_DropDown(object sender, System.EventArgs e)
        {
            cmbSID.Items.AddRange(Strings.ToArray( ));
            // should only be called the first time it drops down
            cmbSID.DropDown -= new System.EventHandler(cmbSID_DropDown);
        }

        #endregion
    }
}
