﻿
namespace HaloMap
{
    public class Tag
    {
        /// <summary>
        /// Gets the Type of the TagClass
        /// </summary>
        public string Type { get; private set; }
        /// <summary>
        /// Gets the Path of the TagClass
        /// </summary>
        public string Name { get; private set; }
        /// <summary>
        /// Gets the Ident of the TagClass
        /// </summary>
        public int Ident { get; private set; }
        /// <summary>
        /// Gets the non-translated Meta Offset of the TagClass
        /// </summary>
        public int RawMetaOffset { get; private set; }
        /// <summary>
        /// Gets the Size of the Meta for the TagClass
        /// </summary>
        public int MetaSize { get; private set; }
        /// <summary>
        /// Gets the Magic used to translate pointers for the TagClass
        /// </summary>
        public int Magic { get; private set; }
        /// <summary>
        /// Gets the translated Meta Offset for the TagClass
        /// </summary>
        public int MetaOffset { get { return RawMetaOffset - Magic; } }
        
        /// <summary>
        /// Initializes a new instance of TagClass
        /// </summary>
        /// <param name="ident">The identifier of the TagClass</param>
        /// <param name="rawMetaOffset">The pure meta offset(untranslated) for the TagClass</param>
        /// <param name="metaSize">The size of the TagClass</param>
        /// <param name="magic">The magic value for the TagClass</param>
        /// <param name="type">The tagClass type</param>
        /// <param name="name">The name of the tagClass</param>
        public Tag(int ident, int rawMetaOffset, int metaSize, int magic, string type, string name)
        {
            Ident = ident;
            RawMetaOffset = rawMetaOffset;
            MetaSize = metaSize;
            Magic = magic;
            Type = type;
            Name = name;
        }
    }
}
