using System;
using System.Collections.Generic;
using System.Collections;

namespace Utilities
{
	/// <summary>
	/// Summary description for Class1.
	/// </summary>
	public class BiDirectionalHashtable<T1, T2> : ICollection
    {
        #region -    fields     -

        protected Dictionary<T1, T2> keysToItems;
        protected Dictionary<T2, T1> itemsToKeys;

        #endregion
        #region -  constructor  -

        public BiDirectionalHashtable()
		{
            keysToItems = new Dictionary<T1, T2>();
            itemsToKeys = new Dictionary<T2, T1>();
        }

        #endregion
        #region -    methods    -

        public void Clear()
		{
			keysToItems.Clear();
			itemsToKeys.Clear();
		}

		public bool Contains(T2 item)
		{
			return itemsToKeys.ContainsKey(item);
		}

		public bool ContainsKey(T1 key)
		{
			return keysToItems.ContainsKey(key);
		}

		public ICollection Keys
		{
			get
			{
				return keysToItems.Keys;
			}
		}

		public ICollection Values
		{
			get
			{
				return itemsToKeys.Keys;
			}
		}

		public void AddItem(T1 key, T2 item)
		{
            if (keysToItems.ContainsKey(key) || itemsToKeys.ContainsKey(item))
            {
                keysToItems.Remove(key);
                itemsToKeys.Remove(item);
            }
            keysToItems[key] = item;
			itemsToKeys[item] = key;
		}

		public void RemoveByKey(T1 key)
		{
			if (key == null)
				throw new ArgumentNullException("key");

			T2 item = keysToItems[key];

			keysToItems.Remove(key);
			itemsToKeys.Remove(item);
		}

		public void RemoveByItem(T2 item)
		{
			if (item == null)
				throw new ArgumentNullException("item");

			T1 key = itemsToKeys[item];

			keysToItems.Remove(key);
			itemsToKeys.Remove(item);
		}

		public T1 GetKey(T2 item)
		{
			if (item == null)
				throw new ArgumentNullException("keitemy");
            T1 temp;
            if (itemsToKeys.TryGetValue(item, out temp))
            {
                return temp;
            }
            return default(T1);
		}

        public T2 GetItem(T1 key)
		{
			if (key == null)
				throw new ArgumentNullException("key");
            T2 temp;
            if (keysToItems.TryGetValue(key, out temp))
            {
                return temp;
            }
            return default(T2);
        }

        #endregion

        #region ICollection Members

        public int Count
		{
			get
			{
				return keysToItems.Count;
			}
		}

		public void CopyTo(Array array, int index)
		{
			if (array == null)
				throw new ArgumentNullException("array");

			if (index < 0)
				throw new ArgumentOutOfRangeException("index");

			if (array.Rank != 1)
				throw new ArgumentException("array is multidimensional.");
			if (index > array.Length)
				throw new ArgumentException("index is equal to or greater than the length of array.");
			if (array.Length - index < Count)
				throw new ArgumentException("The number of elements in the source BiDirectionalHashtable is greater than the available space from index to the end of the destination array.");

			int i = index;
			foreach (T2 item in itemsToKeys.Keys)
			{
				array.SetValue(item, i);
				i++;
			}
		}

		public object SyncRoot
		{
			get
			{
				return this;
			}
		}

        public bool IsSynchronized
        {
            get { return true; }
        }

		#endregion

		#region IEnumerable Members

		public IEnumerator GetEnumerator()
		{
			return keysToItems.GetEnumerator();
		}

		#endregion
    }
}
