﻿using System.Windows.Forms;
using HaloMap;
using YeloDebug;
using System.Threading;
using System.Net;

namespace Standalone_RTH
{
    public partial class Main : Form
    {
        #region properties

        Xbox xbox { get; set; }
        Map map { get; set; }
        Tag tag { get; set; }
        const int NotifyTimeout = 3000;

        #endregion
        #region constructor

        public Main( )
        {
            InitializeComponent( );
            xbox = new Xbox( );
        }

        #endregion
        #region methods

        void OpenMap(string filename)
        {
            if( map != null )
            {
                map.Close( );
            }
            map = new Map(filename);
            map.Load( );
            Enabled = false;
            map.Tags.LoadTreeView(tv);
            Enabled = true;
        }

        void Open(object sender, System.EventArgs e)
        {
            using( OpenFileDialog ofd = new OpenFileDialog( ) )
            {
                ofd.Filter = "Halo 2 Map File|*.map";
                if( ofd.ShowDialog(this) == DialogResult.OK )
                {
                    // make ofd dissappear fast
                    Application.DoEvents( );
                    OpenMap(ofd.FileName);
                }
            }
        }

        void Connect(object sender, System.EventArgs e)
        {
            IPAddress addy = null;
            if( IPAddress.TryParse(txtXboxIP.Text, out addy) == false )
            {
                Notify("Real Time Halo", "Enter a valid IP address in the Real Time Halo IP TextBox.", ToolTipIcon.Warning);
                return;
            }
            if( xbox.Connected )
            {
                Notify("Real Time Halo", "Disconnect your current connection before starting another.", ToolTipIcon.Warning);
            }
            else
            {
                try
                {
                    xbox.Connect(txtXboxIP.Text.Trim( ));
                    Notify("Real Time Halo", "Sucessfully connected to " + txtXboxIP.Text.Trim( ), ToolTipIcon.Info);
                }
                catch
                {
                    Notify("Real Time Halo", "Failed to connect to " + txtXboxIP.Text.Trim( ), ToolTipIcon.Error);
                }
            }
        }

        void Disconnect(object sender, System.EventArgs e)
        {
            if( xbox.Connected )
                return;
            xbox.Disconnect( );
            Notify("Real Time Halo", "Disconnected", ToolTipIcon.Info);
        }

        void Sync(object sender, System.EventArgs e)
        {
            if( xbox.Connected == false || tag == null || map == null )
                return;
            map.Sync(tag, xbox);
        }

        void TagSelected(object sender, TreeViewEventArgs e)
        {
            tag = map.Tags.FindTag(e.Node);
        }

        #region Notify Icon Code

        delegate void SimpleDelegate( );

        void Notify(string title, string text, ToolTipIcon ttIcon)
        {
            notify.ShowBalloonTip(NotifyTimeout, title, text, ttIcon);
            // thread sleep the amount of time it's supposed to show
            // then toggle the visibility to make it dissappear at the right time
            // do it on a different thread so we don't lock up the UI thread
            Thread t = new Thread(HideBalloonTip);
            t.Start( );
        }
        void HideBalloonTip( )
        {
            System.Threading.Thread.Sleep(NotifyTimeout);
            // invoke it so it runs on the UI thread
            Invoke(new SimpleDelegate(ToggleVisibility));
        }
        void ToggleVisibility( )
        {
            notify.Visible = false;
            notify.Visible = true;
        }

        #endregion

        #endregion
    }
}
